@@ -48,7 +48,7 @@ static double initialAvgFov;   // these three for fov stabilization
 static double avgfovFromSAP;
 static int needInitialAvgFov;
 
-#define ADJUST_LOG_FILENAME "c:\\PToolsLog.txt"  // file name for logging, if enabled
+#define ADJUST_LOG_FILENAME "PToolsLog.txt"  // file name for logging, if enabled
 #define ADJUST_LOGGING_ENABLED 0
 
 
@@ -581,6 +581,8 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
 */
         int image_selection_width=im->width;
         int image_selection_height=im->height;
+	mp->im = im;
+	mp->pn = pn;
         if(im->cP.horizontal)
         {
           mp->horizontal=im->cP.horizontal_params[color];
@@ -675,11 +677,18 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
         case _fisheye_ff:
         case _fisheye_circ:
         case _panorama:
+        case _lambert:
         case _mercator:
+	case _millercylindrical:
         case _sinusoidal:
             // horizontal pixels per degree
             mp->distance        = ((double) pn->width) / b;
             break;
+        case _lambertazimuthal: 
+	    tpara = 1;
+	    lambertazimuthal_erect(b/2.0, 0.0, &tx, &ty, & tpara);
+            mp->distance = pn->width/(2.0*tx);
+            break;
         case _stereographic:
             tpara = 1;
             stereographic_erect(b/2.0, 0.0, &tx, &ty, & tpara);
@@ -690,6 +699,12 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
             transmercator_erect(b/2.0, 0.0, &tx, &ty, &tpara);
             mp->distance = pn->width/(2.0*tx);
             break;
+	case _albersequalareaconic:
+	    mp->distance = 1.0;
+	    //albersequalareaconic_erect(1.924913116, -PI/2.0, &tx, &ty, mp); //b/2.0
+	    albersequalareaconic_distance(&tx, mp);
+	    mp->distance = pn->width/(2.0*tx);
+	    break;
         default:
             // unknown
             PrintError ("SetMakeParams: Unsupported panorama projection");
@@ -727,6 +742,17 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
 	mp->rot[0]		= mp->distance * PI;								// 180 in screenpoints
 	mp->rot[1]		= -im->yaw *  mp->distance * PI / 180.0; 			//    rotation angle in screenpoints
 
+
+	/*
+	printf("Image format %d\n", im->format);
+	printf("shear[0] %f\n", mp->shear[0]);
+	printf("shear[1] %f\n", mp->shear[1]);
+	printf("rot[0] %f\n", mp->rot[0]);
+	printf("rot[1] %f\n", mp->rot[1]);
+	printf("scale[0] %f\n", mp->rot[0]);
+	*/
+
+
 	mp->perspect[0] = (void*)(mp->mt);
 	mp->perspect[1] = (void*)&(mp->distance);
 			
@@ -761,6 +787,18 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
    	else if(pn->format == _mercator)
 	{
 		SetDesc(stack[i],	erect_mercator,		&(mp->distance)	); i++;	// Convert mercator to sphere
+    }
+   	else if(pn->format == _millercylindrical)
+	{
+		SetDesc(stack[i],	erect_millercylindrical,		&(mp->distance)	); i++;	// Convert miller to sphere
+    }
+   	else if(pn->format == _lambert)
+	{
+		SetDesc(stack[i],	erect_lambert,		&(mp->distance)	); i++;	// Convert lambert to sphere
+    }
+   	else if(pn->format == _lambertazimuthal)
+	{
+		SetDesc(stack[i],	erect_lambertazimuthal,		&(mp->distance)	); i++;	// Convert lambert to sphere
     }
    	else if(pn->format == _trans_mercator)
 	{
@@ -774,6 +812,10 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
 	{
 		SetDesc(stack[i],	erect_sinusoidal,		&(mp->distance)	); i++;	// Convert sinusoidal to sphere
 	}
+   	else if(pn->format == _albersequalareaconic)
+	{
+		SetDesc(stack[i],	erect_albersequalareaconic,		mp	); i++;	// Convert albersequalareaconic to sphere
+	}
    	else if(pn->format == _equirectangular) 
 	{
 		// no conversion needed		
@@ -886,12 +928,14 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
 
 	int 		i;
 	double		a,b;							// field of view in rad
-    double      tx,ty,tpara;
+	double      tx,ty,tpara;
 
 	a =	 DEG_TO_RAD( im->hfov );	// field of view in rad		
 	b =	 DEG_TO_RAD( pn->hfov );
 
 
+	mp->im = im;
+	mp->pn = pn;
 	SetMatrix( 	DEG_TO_RAD( im->pitch ), 
 				0.0, 
 				DEG_TO_RAD( im->roll ), 
@@ -908,11 +952,18 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
         case _fisheye_ff:
         case _fisheye_circ:
         case _panorama:
+        case _lambert:
         case _mercator:
+        case _millercylindrical:
         case _sinusoidal:
             // horizontal pixels per degree
             mp->distance        = ((double) pn->width) / b;
             break;
+	case _lambertazimuthal:
+	    tpara = 1;
+	    lambertazimuthal_erect(b/2.0, 0.0, &tx, &ty, & tpara);
+	    mp->distance = pn->width/(2.0*tx);
+            break;
         case _stereographic:
             tpara = 1;
             stereographic_erect(b/2.0, 0.0, &tx, &ty, & tpara);
@@ -923,6 +974,12 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
             transmercator_erect(b/2.0, 0.0, &tx, &ty, & tpara);
             mp->distance = pn->width/(2.0*tx);
             break;
+        case _albersequalareaconic:
+            mp->distance = 1.0;
+            //albersequalareaconic_erect(1.924913116, -PI/2.0, &tx, &ty, mp);  //b/2.0
+	    albersequalareaconic_distance(&tx, mp);
+            mp->distance = pn->width/(2.0*tx);
+            break;
         default:
             // unknown
             PrintError ("SetInvMakeParams: Unsupported panorama projection");
@@ -1073,6 +1130,18 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
    	else if(pn->format == _mercator)
 	{
 		SetDesc(stack[i],	mercator_erect,		&(mp->distance)	); i++;	// Convert sphere to sphere
+    }
+   	else if(pn->format == _millercylindrical)
+	{
+		SetDesc(stack[i],	millercylindrical_erect,		&(mp->distance)	); i++;	// Convert sphere to sphere
+    }
+   	else if(pn->format == _lambert)
+	{
+		SetDesc(stack[i],	lambert_erect,		&(mp->distance)	); i++;	// Convert sphere to lambert
+    }
+   	else if(pn->format == _lambertazimuthal)
+	{
+		SetDesc(stack[i],	lambertazimuthal_erect,		&(mp->distance)	); i++;	// Convert sphere to lambert azimuthal
     }
    	else if(pn->format == _trans_mercator)
 	{
@@ -1086,6 +1155,10 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
 	{
 		SetDesc(stack[i],	sinusoidal_erect,		&(mp->distance)	); i++;	// Convert sphere to sinusoidal
 	}
+   	else if(pn->format == _albersequalareaconic)
+	{
+		SetDesc(stack[i],	albersequalareaconic_erect,		mp	); i++;	// Convert sphere to albersequalareaconic
+	}
    	else if(pn->format == _equirectangular) 
 	{
 		// no conversion needed		
@@ -1539,6 +1612,28 @@ double rectDistSquared( int num )
 	return result;
 }
 
+/// huber() is an M-Estimator function. Using an M-Estimator might
+/// work better if the control points contain outliers (eg. from autopano).
+/// this implementation accepts normal, non squared errors, and return non-squared errors,
+/// contrary to the definition in the literature (where the square is
+/// included in the function)
+
+static double fcnPanoHuberSigma = 0; // sigma for Huber M-estimator. 0 disables M-estimator
+
+void setFcnPanoHuberSigma(double sigma)
+{
+    fcnPanoHuberSigma = sigma;
+}
+
+double huber(double x, double sigma)
+{
+    if (abs(x) < sigma)
+        return x;
+    else
+        return sqrt(2.0*sigma*abs(x) - sigma*sigma);
+}
+
+
 
 /// (function distSquared2 has been removed -- it was unused and redundant)
 
@@ -1759,8 +1854,12 @@ int fcnPano(int m, int n, double x[], double fvec[], int *iflag)
 	for( i=0; i < g->numPts; i++){
 		if (fcnPanoNperCP == 1) {
 			EvaluateControlPointErrorAndComponents ( i, &fvec[iresult], &junk2[0]);
-		} else {
+        } else {
 			EvaluateControlPointErrorAndComponents ( i, &junk, &fvec[iresult]);
+            if (fcnPanoHuberSigma) {
+                fvec[iresult] = huber(fvec[iresult], fcnPanoHuberSigma);
+                fvec[iresult+1] = huber(fvec[iresult+1], fcnPanoHuberSigma);
+            }
 		}
 		
 		// Field-of-view stabilization.  Applying here means that the
@@ -2469,7 +2568,10 @@ int CheckParams( AlignInfo *g )
 	if( g->pano.format != _rectilinear 		&& g->pano.format != _panorama &&
 		g->pano.format != _equirectangular 	&& g->pano.format != _fisheye_ff &&
 		g->pano.format != _stereographic 	&& g->pano.format != _mercator &&
-		g->pano.format != _trans_mercator 	&& g->pano.format != _sinusoidal) 
+		g->pano.format != _trans_mercator 	&& g->pano.format != _sinusoidal &&
+ 	        g->pano.format != _lambert              && g->pano.format != _lambertazimuthal &&
+		g->pano.format != _albersequalareaconic
+	     ) 
 		    	err=11;
 		    
 	// Check Control Points
