@@ -30,7 +30,7 @@
 #include <limits.h>
 
 #include "panorama.h"
-//#include "tiffio.h"
+#include "tiffio.h"
 
 #ifndef TRUE
 	#define TRUE 1
@@ -40,6 +40,10 @@
 	#define FALSE 0
 #endif
 
+#ifndef bzero
+	#define bzero(dest, len)   memset((dest), 0, (len))
+#endif
+
 
 //---------------------- Types ---------------------------------------------
 
@@ -56,7 +60,7 @@ enum{
 //---------------------- Some useful math defines --------------------------
 
 #ifndef PI
-	#define PI 3.14159265358979323846
+        #define PI 3.14159265358979323846264338327950288
 #endif
 #ifndef HALF_PI
     #define HALF_PI (PI*0.5)
@@ -205,8 +209,11 @@ struct stitchBuffer{						// Used describe how images should be merged
 	int				feather;		// Width of feather
 	int				colcorrect;		// Should the images be color corrected?
 	int				seam;			// Where to put the seam (see above)
+        unsigned char                   psdOpacity;               // Opacity of the layer. Currently used only by PSD output. 0 trans, 255 opaque
+        unsigned char                   psdBlendingMode;          // blending mode (photoshop)
 	};
 
+
 typedef struct stitchBuffer stBuf;
 
 struct panControls{							// Structure for realtime Panoeditor
@@ -351,6 +358,8 @@ struct MakeParams{								// Actual parameters used by Xform functions for pano-
 	double  distance;
 	double	horizontal;
 	double	vertical;
+	Image *im;
+	Image *pn;
 	};
 
 struct LMStruct{								// Parameters used by the Levenberg Marquardt-Solver
@@ -640,16 +649,21 @@ int 	GetFullPath 		(fullPath *path, char *filename); // Somewhat confusing, for
 int 	StringtoFullPath	(fullPath *path, char *filename);
 int 	IsTextFile			( char* fname );
 int 	readPositions		( char* script, transformCoord *tP );
-int 	readImage			( Image *im, fullPath *sfile );
 int	readJPEG			( Image *im, fullPath *sfile );
 int	readTIFF			( Image *im, fullPath *sfile );
-int 	writeImage			( Image *im, fullPath *sfile );
 int 	writeJPEG			( Image *im, fullPath *sfile, 	int quality, int progressive );
-int 	makeTempPath		( fullPath *path );
 int 	writePNG			( Image *im, fullPath *sfile );
 int 	readPNG				( Image *im, fullPath *sfile );
 int 	LaunchAndSendScript(char* application, char* script);
 aPrefs* readAdjustLine( fullPath *theScript );
+
+#ifdef __Mac__
+
+ int 	readImage			( Image *im, fullPath *sfile );
+ int 	writeImage			( Image *im, fullPath *sfile );
+ int 	makeTempPath		( fullPath *path );
+#endif
+
 //int	readtif(Image *im, TIFF* tif);
 void getCropInformation(char *filename, CropInfo *c);
 
@@ -715,6 +729,11 @@ void 	nextWord			( register char* word, char** ch );
 void 	nextLine			( register char* line, char** ch );
 int 	numLines			( char* script, char first );
 
+char *panoParserFindOLine(char *script, int index);
+
+
+
+
 // Memory
 
 void 	DisposeAlignInfo	( AlignInfo *g );
@@ -814,12 +833,22 @@ int erect_sphere_tp	( double x_dest,double  y_dest, double* x_src, double* y_src
 int mirror_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 int mercator_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 int erect_mercator		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
+int lambert_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
+int erect_lambert		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
+int erect_lambertazimuthal( double x_dest,double  y_dest, double* x_src, double* y_src, void* params);
+int lambertazimuthal_erect( double x_dest,double  y_dest, double* x_src, double* y_src, void* params);
 int transmercator_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 int erect_transmercator		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 int sinusoidal_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 int erect_sinusoidal		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 int stereographic_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 int erect_stereographic		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
+int albersequalareaconic_erect	( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
+int erect_albersequalareaconic	( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
+int albersequalareaconic_distance	( double *x_src, void* params );
+int millercylindrical_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
+int erect_millercylindrical		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
+
 
 int mirror_sphere_cp	( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
 int mirror_pano		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
@@ -891,6 +920,25 @@ extern sPrefs			*gsPrPtr;
 
 #endif // PT_BIGENDIAN
 
+//TODO:  JMW These File i/o macros are to be replaced in code with the error catching functions below
+#define WRITEUCHAR( theChar )       ch = theChar; count = 1; mywrite(fnum,count,&ch);
+
+#define WRITESHORT( theShort )      svar = theShort; d = data; SHORTNUMBER( svar, d ); \
+                                    count = 2; mywrite  (fnum,count,data);
+
+#define WRITEINT32( theLong )       var = theLong; d = data; LONGNUMBER( var, d ); \
+                                    count = 4; mywrite  (fnum,count,data);
+
+#define READINT32( theLong )                count = 4; myread(src,count,data);  \
+                                            d = data; NUMBERLONG( var, d );     \
+                                            theLong = var;
+                                    
+#define READSHORT( theShort )               count = 2; myread(src,count,data);  \
+                                            d = data; NUMBERSHORT( svar, d );   \
+                                            theShort = svar;
+
+#define READUCHAR( theChar )                count = 1; myread(src,count,&ch); theChar = ch; 
+
 // Cross platform file functions
 
 #ifdef __Mac__
@@ -939,9 +987,49 @@ extern sPrefs			*gsPrPtr;
 
 #endif
 
+/* ENDIAN aware file i/o funtions.  Used for reading and writing photoshop files */
+Boolean panoWriteUCHAR(file_spec fnum, UCHAR   theChar );
+Boolean panoWriteSHORT(file_spec fnum, USHORT  theShort );
+Boolean panoWriteINT32(file_spec fnum, ULONG   theLong );
+Boolean panoReadUCHAR (file_spec fnum, UCHAR  *pChar );
+Boolean panoReadSHORT (file_spec fnum, USHORT *pShort );
+Boolean panoReadINT32 (file_spec fnum, ULONG  *pLong );
+    
 
+#define PANO_DEFAULT_PIXELS_PER_RESOLUTION  150.0
+#define PANO_DEFAULT_TIFF_RESOLUTION_UNITS  RESUNIT_INCH
+// this is the best compression available in all systems
+// better than PACKBITS
+#define PANO_DEFAULT_TIFF_COMPRESSION       COMPRESSION_DEFLATE
+
+void panoMetadataFree(pano_ImageMetadata * metadata);
+int panoMetadataCopy(pano_ImageMetadata * to, pano_ImageMetadata * from);
+int panoROIRowInside(pano_CropInfo * cropInfo, int row);
+void panoMetadataSetCompression(pano_ImageMetadata * metadata, char *compressionName);
+int panoMetadataCopy(pano_ImageMetadata * to, pano_ImageMetadata * from);
+void panoMetadataFree(pano_ImageMetadata * metadata);
+
+void panoMetadataSetAsCropped(pano_ImageMetadata * metadata, 
+			      int croppedWidth, 
+			      int croppedHeight,
+			      int roiLeft, 
+			      int roiRight);
+
+void panoMetadataResetSize(pano_ImageMetadata * metadata, 
+			   int width, int height);
+
+int panoReadJPEG(Image * im, fullPath * sfile);
+
+#ifndef PAN_DEBUG_METADATA
+void panoDumpMetadata(pano_ImageMetadata * metadata, char *message);
+#else 
+#define panoDumpMetadata(a,b)  ;
+#endif
 
 
 #endif
 
 
+// number of different temporary files
+#define MAX_TEMP_TRY    1000000
+
