@@ -21,11 +21,11 @@
 #include "panotypes.h"
 #include <signal.h>
 
-#define PREF_FILE_NAME "pano13.prf"
 
 //------------------ Public functions required by filter.h -------------------------------
 
 
+
 void filter_main( TrformStr *TrPtr, struct size_Prefs *spref)
 {
 	dispatch	( TrPtr, spref);
@@ -38,15 +38,29 @@ void filter_main( TrformStr *TrPtr, struct size_Prefs *spref)
 
 void  PrintErrorIntern( char* fmt, va_list ap)
 {
-	char message[257];
-	
-	vsprintf(message, fmt, ap);
-	
+	char message[512];
+	char *toPrint;
+
+	if (strlen(fmt) < 512) {
+ 	  vsprintf(message, fmt, ap);	  
+	  toPrint = message;
+	}   else {
+	  // we don't have enough space, so just 
+	  // print original string
+	  toPrint = fmt;
+	}
+
 	if( JavaUI ){
-		JPrintError( message );
+	  JPrintError( toPrint );
 	}else{
-		printf("%s\n", message);
-		fflush(stdout);
+	  printf(toPrint);
+
+	  // Add an end of line if none is provide
+	  if (strlen(toPrint) > 0 &&
+	      toPrint[strlen(toPrint)-1] != '\n') {
+	      putchar('\n');
+	  }
+	  fflush(stdout);
 	}
 }
 
@@ -137,7 +151,7 @@ int readPrefs( char* pref, int selector )
 		struct size_Prefs			s;
 		panControls					pc;
 	} prf;
-	char* prefname = PREF_FILE_NAME;
+	char* prefname = "pano12.prf";
 	long size;
 
 	FILE 	*prfile;
@@ -222,7 +236,7 @@ void writePrefs( char* prefs, int selector ){
 	} prf;
 
 	FILE 	*prfile;
-	char* prefname = PREF_FILE_NAME;
+	char* prefname = "pano12.prf";
 
 
 
