@@ -71,8 +71,6 @@ caused annoying warnings about unknown tags while loading each tiff source image
 #define ID_MM  100
 #define ID_SM  101
 
-#define PREF_FILE_NAME "pano13.prf"
-
 HINSTANCE 	hDllInstance 	= NULL;
 HWND 		wndOwner 		= NULL;
 int 		dialogDone;
@@ -298,11 +296,11 @@ int readPrefs( char* pref, int selector )
 	if( c!= NULL )
 	{
 		c++;
-		strcpy( c,PREF_FILE_NAME );
+		strcpy( c,"pano12.prf" );
 	}
 	else
 	{
-		strcpy( prefname,".\\"PREF_FILE_NAME );
+		strcpy( prefname,".\\pano12.prf" );
 	}
 
 
@@ -398,11 +396,11 @@ void writePrefs( char* prefs, int selector )
 	if( c!= NULL )
 	{
 		c++;
-		strcpy( c,PREF_FILE_NAME );
+		strcpy( c,"pano12.prf" );
 	}
 	else
 	{
-		strcpy( prefname,".\\"PREF_FILE_NAME );
+		strcpy( prefname,".\\pano12.prf" );
 	}
 
 
