@@ -25,28 +25,29 @@
 #define VERS2 0x00
 #endif
 
-// version of preferences file used by plug-ins, used to verify data, Only updated when data structure changes
+//version of preferences file, used to verify data
 #ifndef PREF_VERSION
-#define PREF_VERSION "2.8.5 "
+#define PREF_VERSION "2.9.12 "
 #endif
 
 // String style of global version
 #ifndef VERSION
-#define VERSION "2.8.6 "
+#define VERSION "2.9.12 "
 #endif 
 
 // Numeric style of global version, same as VERSION with more precision
 #ifndef PTVERSION_FILEVERSIONNUMBER
-#define PTVERSION_FILEVERSIONNUMBER 2,8,6,0
+#define PTVERSION_FILEVERSIONNUMBER 2,9,7,-1
 #endif
 
 #ifndef PTVERSION_NAME_LONG
 #define PTVERSION_NAME_LONG "LongVersion"
 #endif
 #ifndef LONGVERSION
-#define LONGVERSION VERSION ", Copyright (c) 1998-2001, H. Dersch, der@fh-furtwangen.de"
+#define LONGVERSION VERSION ", Copyright (c) 1998-2006, H. Dersch, der@fh-furtwangen.de"
 #endif
 
+
 #ifndef PTVERSION_NAME_FILEVERSION
 #define PTVERSION_NAME_FILEVERSION "FileVersion"
 #endif
@@ -62,16 +63,16 @@
 #define PTVERSION_NAME_LEGALCOPYRIGHT "LegalCopyright"
 #endif
 #ifndef PTVERSION_LEGALCOPYRIGHT 
-#define PTVERSION_LEGALCOPYRIGHT "Copyright  1999, 2000, 2001 Helmut Dersch\0"
+#define PTVERSION_LEGALCOPYRIGHT "Copyright  1999, 2000, 2001, 2005, 2006 Helmut Dersch\0"
 #endif
 
 #ifndef PTVERSION_NAME_COMMENT
 #define PTVERSION_NAME_COMMENT "Comments"
 #endif
 #ifndef PTVERSION_COMMENT
-  #ifdef _MSVC
-    #define PTVERSION_COMMENT "MSVC\0"
-  #else
-    #define PTVERSION_COMMENT "MinGW\0"
-  #endif
+# ifdef _MSC_VER
+#  define PTVERSION_COMMENT "MSVC\0"
+# else
+#  define PTVERSION_COMMENT "MinGW\0"
+# endif
 #endif
